﻿using System;
using System.Collections.Generic;
using System.Linq;

class Program
{
    static void Main(string[] args)
    {
        MyClass mc = new MyClass(Enumerable.Range(0, 10));

        var items = mc[(elem) => elem < 5];
        Output(items); // 0, 1, 2, 3, 4,

        // python - mc[:3]
        Output(mc[..3]); // 0, 1, 2

        // python - mc[1:3]
        Output(mc[1..3]); // 1, 2

        // python - mc[5:]
        Output(mc[5..]); // 5, 6, 7, 8, 9

        // python - mc[5:-1]
        Output(mc[5..^1]); // 5, 6, 7, 8

        // python - mc[:]
        Output(mc[..]); // 0, 1, 2, 3, 4, 5, 6, 7, 8, 9

        void Output(IEnumerable<int> e)
        {
            foreach (var item in e)
            {
                Console.Write($"{item}, ");
            }

            Console.WriteLine();
        }
    }
}

public class MyClass
{
    List<int> _list = new List<int>();

    public MyClass(IEnumerable<int> seq)
    {
        _list.AddRange(seq);
    }

    public IEnumerable<int> this[Range range] // [range.Start, range.End) 범위 선택
    {
        get 
        {
            (int offset, int take) = range.GetOffsetAndLength(_list.Count);

            /*
            int offset = range.Start.Value;

            int take = range.End.Value - range.Start.Value;
            if (range.End.IsFromEnd == true)
            {
                take = (_list.Count - range.End.Value - range.Start.Value);
            }
            */

            return _list.Skip(offset).Take(take);
        }
    }

    public IEnumerable<int> this[Func<int, bool> condition]
    {
        get
        {
            return _list.Where(condition);
        }
    }
}
